using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace PhonePiano
{
    /// <summary>
    /// Creates a simple piano that you can use to impress your friends.
    /// Play notes and chords with the keys, or touch the top right hand corner
    /// for an impressive "solo".
    /// 
    /// Touch the top left hand corner to leave the game.
    /// 
    /// Rob Miles 2010 
    /// www.robmiles.com
    /// 
    /// </summary>
    public class PianoGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        /// <summary>
        /// Note names are used to identify the sound samples to be loaded. Given in order
        /// up the keyboard
        /// </summary>
        string[] noteNames = new string[] {
            "c1","csharp", "d","dsharp", "e", "f","fsharp", "g","gsharp", "a","asharp", "b", "c2"
        };

        /// <summary>
        /// These are the keys on the keyboard
        /// </summary>
        class keyDetails
        {
            /// <summary>
            /// Texture to draw for the key.
            /// </summary>
            public Texture2D keyTexture;
            /// <summary>
            /// Position on the screen for this key
            /// </summary>
            public Rectangle keyRectangle;
            /// <summary>
            /// Sound sample to play for the key
            /// </summary>
            public SoundEffect keySound;
        }

        /// <summary>
        /// All the keys on the screen
        /// </summary>
        List<keyDetails> noteKeys = new List<keyDetails>();

        /// <summary>
        /// Cheat song...
        /// </summary>
        Song piano;

        float displayWidth ;
        float displayHeight ;


        public PianoGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromSeconds(1 / 30.0);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            displayWidth = GraphicsDevice.Viewport.Width;
            displayHeight = GraphicsDevice.Viewport.Height;
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Work out the height of each key
            float keyHeight = displayHeight / 8;

            // Going to draw keys from the top of the screen down, as the 
            // notes are in ascending order and the lowest note must be on 
            // the left hand edge

            // Each time we move on to the next key we will drop these values by 
            // the height of the key

            float keyTop = displayHeight - keyHeight;

            foreach (string noteName in noteNames)
            {
                keyDetails newKey = new keyDetails();
                newKey.keySound = Content.Load<SoundEffect>(noteName);

                if (noteName.Contains("sharp"))
                {
                    // Draw the black keys across half of the screen offset from 
                    // the white keys
                    newKey.keyRectangle = new Rectangle(
                        0, (int)((keyTop + (keyHeight / 2)) + 0.5f),
                        (int)(displayWidth / 2), (int)(keyHeight + 0.5f));
                    newKey.keyTexture = Content.Load<Texture2D>("BlackKey");

                    // Insert black keys at the head of the key list
                    noteKeys.Insert(0, newKey);
                }
                else
                {
                    // Draw the white key across the whole screen
                    // using the white key texture
                    newKey.keyRectangle = new Rectangle(
                        0, (int)(keyTop + 0.5f),
                        (int)displayWidth, (int)(keyHeight + 0.5f));
                    newKey.keyTexture = Content.Load<Texture2D>("Key");

                    // Move down to the next key on the screen
                    keyTop = keyTop - keyHeight;

                    // Insert the white keys into the end of the key list
                    // This is because black keys are "on top" of the white
                    // ones and we need to find and play them first

                    if (noteKeys.Count > 0)
                    {
                        noteKeys.Insert(noteKeys.Count - 1, newKey);
                    }
                    else
                    {
                        noteKeys.Add(newKey);
                    }
                }
            }

            // Load the "cheat" song
            piano = Content.Load<Song>("piano");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Get the current touch positions
            TouchCollection touchState = TouchPanel.GetState();

            foreach (TouchLocation touch in touchState)
            {
                Point touchPoint =
                    new Point((int)touch.Position.X, (int)touch.Position.Y);
                foreach (keyDetails key in noteKeys)
                {
                    if (key.keyRectangle.Contains(touchPoint) &&
                        touch.State == TouchLocationState.Pressed)
                    {
                        key.keySound.Play();
                        break;
                    }
                }

                // If we touch the top right hand corner we play the cheat song
                if (touchPoint.X < 50 && touchPoint.Y < 50)
                {
                    MediaPlayer.Play(piano);
                }
                if (touchPoint.X < 50 && touchPoint.Y < 50)
                {
                    MediaPlayer.Play(piano);
                }

                if (touchPoint.X < 50 && touchPoint.Y > displayHeight- 50)
                {
                    Exit();
                }
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();

            // Draw from the note array so that the black keys are 
            // drawn on top of the white ones

            for (int i = noteKeys.Count - 1; i >= 0; i--)
            {
                keyDetails key = noteKeys[i];
                spriteBatch.Draw(key.keyTexture, key.keyRectangle, Color.White);
            }

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
